@@ -441,10 +441,16 @@ class Resource(object):
         """
         if self.id is None or self.action == self.INIT:
             raise exception.ResourceNotAvailable(resource_name=self.name)
-        LOG.debug('Setting metadata for %s', six.text_type(self))
         refresh = merge_metadata is not None
         db_res = resource_objects.Resource.get_obj(self.stack.context, self.id,
                                                    refresh=refresh)
+        if db_res.action == self.DELETE:
+            self._db_res_is_deleted = True
+            LOG.debug("resource %(name)s, id: %(id)s is DELETE_%(st)s, "
+                      "not setting metadata",
+                      {'name': self.name, 'id': self.id, 'st': db_res.status})
+            raise exception.ResourceNotAvailable(resource_name=self.name)
+        LOG.debug('Setting metadata for %s', six.text_type(self))
         if refresh:
             metadata = merge_metadata(metadata, db_res.rsrc_metadata)
         db_res.update_metadata(metadata)
@@ -2146,6 +2152,9 @@ class Resource(object):
             # Don't log an event as it just spams the user.
             pass
         except Exception as ex:
+            if hasattr(self, '_db_res_is_deleted'):
+                # No spam required
+                return
             LOG.info(_LI('signal %(name)s : %(msg)s'),
                      {'name': six.text_type(self),
                       'msg': six.text_type(ex)},
@@ -2163,11 +2172,12 @@ class Resource(object):
         signal. The base-class raise an exception if no handler is implemented.
         """
         if need_check:
-            self._signal_check_action()
             self._signal_check_hook(details)
         if details and 'unset_hook' in details:
             self._unset_hook(details)
             return False
+        if need_check:
+            self._signal_check_action()
         self._handle_signal(details)
         return self.signal_needs_metadata_updates
 
