@@ -791,8 +791,7 @@ class Stack(collections.Mapping):
         return handler and handler(resource_name)
 
     @profiler.trace('Stack.validate', hide_args=False)
-    def validate(self, ignorable_errors=None, validate_by_deps=True,
-                 validate_resources=True):
+    def validate(self, ignorable_errors=None, validate_by_deps=True):
         """Validates the stack."""
         # TODO(sdake) Should return line number of invalid reference
 
@@ -839,39 +838,36 @@ class Stack(collections.Mapping):
         else:
             iter_rsc = six.itervalues(resources)
 
-        # Validate resources only for top-level stacks. Nested stacks have
-        # already had their resources validated by their parent.
-        if validate_resources:
-            unique_defns = set(res.t for res in six.itervalues(resources))
-            unique_defn_names = set(defn.name for defn in unique_defns)
+        unique_defns = set(res.t for res in six.itervalues(resources))
+        unique_defn_names = set(defn.name for defn in unique_defns)
 
-            for res in iter_rsc:
-                # Don't validate identical definitions multiple times
-                if res.name not in unique_defn_names:
-                    continue
-                try:
-                    if self.resource_validate:
-                        if res.external_id is not None:
-                            res.validate_external()
-                            continue
-                        result = res.validate()
-                    elif res.external_id is None:
-                        result = res.validate_template()
-                except exception.HeatException as ex:
-                    LOG.debug('%s', ex)
-                    if ignorable_errors and ex.error_code in ignorable_errors:
-                        result = None
-                    else:
-                        raise
-                except AssertionError:
+        for res in iter_rsc:
+            # Don't validate identical definitions multiple times
+            if res.name not in unique_defn_names:
+                continue
+            try:
+                if self.resource_validate:
+                    if res.external_id is not None:
+                        res.validate_external()
+                        continue
+                    result = res.validate()
+                elif res.external_id is None:
+                    result = res.validate_template()
+            except exception.HeatException as ex:
+                LOG.debug('%s', ex)
+                if ignorable_errors and ex.error_code in ignorable_errors:
+                    result = None
+                else:
                     raise
-                except Exception as ex:
-                    LOG.info(_LI("Exception in stack validation"),
-                             exc_info=True)
-                    raise exception.StackValidationFailed(
-                        message=encodeutils.safe_decode(six.text_type(ex)))
-                if result:
-                    raise exception.StackValidationFailed(message=result)
+            except AssertionError:
+                raise
+            except Exception as ex:
+                LOG.info(_LI("Exception in stack validation"),
+                         exc_info=True)
+                raise exception.StackValidationFailed(
+                    message=encodeutils.safe_decode(six.text_type(ex)))
+            if result:
+                raise exception.StackValidationFailed(message=result)
 
         for op_name, output in six.iteritems(self.outputs):
             try:
@@ -929,7 +925,7 @@ class Stack(collections.Mapping):
 
         if self.convergence and action in (
                 self.UPDATE, self.DELETE, self.CREATE,
-                self.ADOPT, self.ROLLBACK):
+                self.ADOPT, self.ROLLBACK, self.RESTORE):
             # if convergence and stack operation is create/update/rollback/
             # delete, stack lock is not used, hence persist state
             updated = self._persist_state()
@@ -947,7 +943,7 @@ class Stack(collections.Mapping):
         # or action == UPDATE/DELETE/ROLLBACK. Else, it would
         # be done before releasing the stack lock.
         if status == self.IN_PROGRESS or action in (
-                self.UPDATE, self.DELETE, self.ROLLBACK):
+                self.UPDATE, self.DELETE, self.ROLLBACK, self.RESTORE):
             self._persist_state()
 
     def _persist_state(self):
@@ -1292,14 +1288,6 @@ class Stack(collections.Mapping):
         LOG.info(_LI('convergence_dependencies: %s'),
                  self.convergence_dependencies)
 
-        # Delete all the snapshots before starting delete operation
-        if self.action == self.DELETE:
-            snapshots = snapshot_object.Snapshot.get_all(self.context,
-                                                         self.id)
-            for snapshot in snapshots:
-                self.delete_snapshot(snapshot)
-                snapshot_object.Snapshot.delete(self.context, snapshot.id)
-
         # create sync_points for resources in DB
         for rsrc_id, is_update in self.convergence_dependencies:
             sync_point.create(self.context, rsrc_id,
@@ -1780,11 +1768,7 @@ class Stack(collections.Mapping):
                                'Failed to %s : %s' % (action, failure))
                 return
 
-        snapshots = snapshot_object.Snapshot.get_all(self.context,
-                                                     self.id)
-        for snapshot in snapshots:
-            self.delete_snapshot(snapshot)
-            snapshot_object.Snapshot.delete(self.context, snapshot.id)
+        self.delete_all_snapshots()
 
         if not backup:
             try:
@@ -1896,6 +1880,13 @@ class Stack(collections.Mapping):
             pre_completion_func=save_snapshot_func)
         sus_task(timeout=self.timeout_secs())
 
+    def delete_all_snapshots(self):
+        """Remove all snapshots for this stack."""
+        snapshots = snapshot_object.Snapshot.get_all(self.context, self.id)
+        for snapshot in snapshots:
+            self.delete_snapshot(snapshot)
+            snapshot_object.Snapshot.delete(self.context, snapshot.id)
+
     @profiler.trace('Stack.delete_snapshot', hide_args=False)
     def delete_snapshot(self, snapshot):
         """Remove a snapshot from the backends."""
@@ -1905,14 +1896,7 @@ class Stack(collections.Mapping):
                 data = snapshot.data['resources'].get(name)
                 scheduler.TaskRunner(rsrc.delete_snapshot, data)()
 
-    @profiler.trace('Stack.restore', hide_args=False)
-    @reset_state_on_error
-    def restore(self, snapshot):
-        """Restore the given snapshot.
-
-        Invokes handle_restore on all resources.
-        """
-        self.updated_time = oslo_timeutils.utcnow()
+    def restore_data(self, snapshot):
         env = environment.Environment(snapshot.data['environment'])
         files = snapshot.data['files']
         template = tmpl.Template(snapshot.data['template'],
@@ -1932,6 +1916,17 @@ class Stack(collections.Mapping):
 
         newstack.parameters.set_stack_id(self.identifier())
 
+        return newstack, template
+
+    @reset_state_on_error
+    def restore(self, snapshot):
+        """Restore the given snapshot.
+
+        Invokes handle_restore on all resources.
+        """
+        self.updated_time = oslo_timeutils.utcnow()
+        newstack = self.restore_data(snapshot)[0]
+
         updater = scheduler.TaskRunner(self.update_task, newstack,
                                        action=self.RESTORE)
         updater()
