@@ -12,15 +12,18 @@
 
 import uuid
 
-from eventlet import event as grevent
+import eventlet.queue
 import mock
 from oslo_config import cfg
 from oslo_messaging.rpc import dispatcher
 import six
 
+from heat.common import environment_util as env_util
 from heat.common import exception
 from heat.common import messaging
+from heat.common import service_utils
 from heat.common import template_format
+from heat.db.sqlalchemy import api as db_api
 from heat.engine.clients.os import glance
 from heat.engine.clients.os import nova
 from heat.engine import environment
@@ -64,8 +67,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                                     return_value=stk.env)
 
         mock_validate = self.patchobject(stk, 'validate', return_value=None)
-        event_mock = mock.Mock()
-        self.patchobject(grevent, 'Event', return_value=event_mock)
+        msgq_mock = mock.Mock()
+        self.patchobject(eventlet.queue, 'LightQueue', return_value=msgq_mock)
 
         # do update
         api_args = {'timeout_mins': 60}
@@ -76,8 +79,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         self.assertEqual(old_stack.identifier(), result)
         self.assertIsInstance(result, dict)
         self.assertTrue(result['stack_id'])
-        self.assertEqual([event_mock], self.man.thread_group_mgr.events)
-        mock_tmpl.assert_called_once_with(template, files=None, env=stk.env)
+        self.assertEqual([msgq_mock], self.man.thread_group_mgr.msg_queues)
+        mock_tmpl.assert_called_once_with(template, files=None)
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
@@ -116,9 +119,10 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         self.patchobject(templatem, 'Template', return_value=stk.t)
         self.patchobject(environment, 'Environment', return_value=stk.env)
         self.patchobject(stk, 'validate', return_value=None)
-        self.patchobject(grevent, 'Event', return_value=mock.Mock())
+        self.patchobject(eventlet.queue, 'LightQueue',
+                         return_value=mock.Mock())
 
-        mock_merge = self.patchobject(self.man, '_merge_environments')
+        mock_merge = self.patchobject(env_util, 'merge_environments')
 
         # Test
         environment_files = ['env_1']
@@ -127,7 +131,64 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                               environment_files=environment_files)
 
         # Verify
-        mock_merge.assert_called_once_with(environment_files, None, params)
+        mock_merge.assert_called_once_with(environment_files, None,
+                                           params, mock.ANY)
+
+    def test_stack_update_nested(self):
+        stack_name = 'service_update_nested_test_stack'
+        parent_stack = tools.get_stack(stack_name + '_parent', self.ctx)
+        owner_id = parent_stack.store()
+        old_stack = tools.get_stack(stack_name, self.ctx,
+                                    owner_id=owner_id, nested_depth=1,
+                                    user_creds_id=parent_stack.user_creds_id)
+        sid = old_stack.store()
+        old_stack.set_stack_user_project_id('1234')
+        s = stack_object.Stack.get_by_id(self.ctx, sid)
+
+        stk = tools.get_stack(stack_name, self.ctx)
+        tmpl_id = stk.t.store(self.ctx)
+
+        # prepare mocks
+        mock_stack = self.patchobject(stack, 'Stack', return_value=stk)
+        mock_load = self.patchobject(stack.Stack, 'load',
+                                     return_value=old_stack)
+        mock_tmpl = self.patchobject(templatem.Template, 'load',
+                                     return_value=stk.t)
+
+        mock_validate = self.patchobject(stk, 'validate', return_value=None)
+        msgq_mock = mock.Mock()
+        self.patchobject(eventlet.queue, 'LightQueue', return_value=msgq_mock)
+
+        # do update
+        api_args = {'timeout_mins': 60}
+        result = self.man.update_stack(self.ctx, old_stack.identifier(),
+                                       None, None, None, api_args,
+                                       template_id=tmpl_id)
+
+        # assertions
+        self.assertEqual(old_stack.identifier(), result)
+        self.assertIsInstance(result, dict)
+        self.assertTrue(result['stack_id'])
+        self.assertEqual([msgq_mock], self.man.thread_group_mgr.msg_queues)
+        mock_tmpl.assert_called_once_with(self.ctx, tmpl_id)
+        mock_stack.assert_called_once_with(
+            self.ctx, stk.name, stk.t,
+            convergence=False,
+            current_traversal=old_stack.current_traversal,
+            prev_raw_template_id=None,
+            current_deps=None,
+            disable_rollback=True,
+            nested_depth=1,
+            owner_id=owner_id,
+            parent_resource=None,
+            stack_user_project_id='1234',
+            strict_validate=True,
+            tenant_id='test_tenant_id',
+            timeout_mins=60,
+            user_creds_id=u'1',
+            username='test_username')
+        mock_load.assert_called_once_with(self.ctx, stack=s)
+        mock_validate.assert_called_once_with()
 
     def test_stack_update_existing_parameters(self):
         # Use a template with existing parameters, then update the stack
@@ -163,6 +224,65 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                              tmpl.env.params)
             self.assertEqual(stk.identifier(), result)
 
+    def test_stack_update_existing_encrypted_parameters(self):
+        # Create the stack with encryption enabled
+        # On update encrypted_param_names should be used from existing stack
+        hidden_param_template = u'''
+heat_template_version: 2013-05-23
+parameters:
+   param2:
+     type: string
+     description: value2.
+     hidden: true
+resources:
+   a_resource:
+       type: GenericResourceType
+'''
+        cfg.CONF.set_override('encrypt_parameters_and_properties', True,
+                              enforce_type=True)
+
+        stack_name = 'service_update_test_stack_encrypted_parameters'
+        t = template_format.parse(hidden_param_template)
+        env1 = environment.Environment({'param2': 'bar'})
+        stk = stack.Stack(self.ctx, stack_name,
+                          templatem.Template(t, env=env1))
+        stk.store()
+        stk.set_stack_user_project_id('1234')
+
+        # Verify that hidden parameters are stored encrypted
+        db_tpl = db_api.raw_template_get(self.ctx, stk.t.id)
+        db_params = db_tpl.environment['parameters']
+        self.assertEqual('cryptography_decrypt_v1', db_params['param2'][0])
+        self.assertNotEqual("foo", db_params['param2'][1])
+
+        # Verify that loaded stack has decrypted paramters
+        loaded_stack = stack.Stack.load(self.ctx, stack_id=stk.id)
+        params = loaded_stack.t.env.params
+        self.assertEqual('bar', params.get('param2'))
+
+        update_params = {'encrypted_param_names': [],
+                         'parameter_defaults': {},
+                         'event_sinks': [],
+                         'parameters': {},
+                         'resource_registry': {'resources': {}}}
+        api_args = {rpc_api.PARAM_TIMEOUT: 60,
+                    rpc_api.PARAM_EXISTING: True}
+
+        with mock.patch('heat.engine.stack.Stack') as mock_stack:
+            stk.update = mock.Mock()
+            mock_stack.load.return_value = loaded_stack
+            mock_stack.validate.return_value = None
+            result = self.man.update_stack(self.ctx, stk.identifier(),
+                                           t,
+                                           update_params,
+                                           None, api_args)
+            tmpl = mock_stack.call_args[0][2]
+            self.assertEqual({u'param2': u'bar'}, tmpl.env.params)
+            # encrypted_param_names must be passed from existing to new
+            # stack otherwise the updated stack won't decrypt the params
+            self.assertEqual([u'param2'], tmpl.env.encrypted_param_names)
+            self.assertEqual(stk.identifier(), result)
+
     def test_stack_update_existing_parameters_remove(self):
         """Test case for updating stack with changed parameters.
 
@@ -235,7 +355,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stk = utils.parse_stack(t, stack_name=stack_name, params=intial_params,
                                 files=initial_files)
         stk.set_stack_user_project_id('1234')
-        self.assertEqual(intial_params, stk.t.env.user_env_as_dict())
+        self.assertEqual(intial_params, stk.t.env.env_as_dict())
 
         expected_reg = {'OS::Foo': 'foo.yaml',
                         'OS::Foo2': 'newfoo2.yaml',
@@ -264,9 +384,9 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                                            api_args)
             tmpl = mock_stack.call_args[0][2]
             self.assertEqual(expected_env,
-                             tmpl.env.user_env_as_dict())
+                             tmpl.env.env_as_dict())
             self.assertEqual(expected_files,
-                             tmpl.files)
+                             tmpl.files.files)
             self.assertEqual(stk.identifier(), result)
 
     def test_stack_update_existing_parameter_defaults(self):
@@ -308,7 +428,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                                            None, api_args)
             tmpl = mock_stack.call_args[0][2]
             self.assertEqual(expected_env,
-                             tmpl.env.user_env_as_dict())
+                             tmpl.env.env_as_dict())
             self.assertEqual(stk.identifier(), result)
 
     def test_stack_update_reuses_api_params(self):
@@ -343,7 +463,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         self.assertTrue(result['stack_id'])
 
         mock_validate.assert_called_once_with()
-        mock_tmpl.assert_called_once_with(template, files=None, env=stk.env)
+        mock_tmpl.assert_called_once_with(template, files=None)
         mock_env.assert_called_once_with(params)
         mock_load.assert_called_once_with(self.ctx, stack=s)
         mock_stack.assert_called_once_with(
@@ -366,8 +486,10 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stk.disable_rollback = False
         stk.store()
 
+        self.man.engine_id = service_utils.generate_engine_id()
+
         self.patchobject(stack.Stack, 'load', return_value=stk)
-        self.patchobject(stack_lock.StackLock, 'try_acquire',
+        self.patchobject(stack_lock.StackLock, 'get_engine_id',
                          return_value=self.man.engine_id)
         self.patchobject(self.man.thread_group_mgr, 'send')
 
@@ -384,9 +506,9 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stk.disable_rollback = False
         stk.store()
         self.patchobject(stack.Stack, 'load', return_value=stk)
-        self.patchobject(stack_lock.StackLock, 'try_acquire',
+        self.patchobject(stack_lock.StackLock, 'get_engine_id',
                          return_value=str(uuid.uuid4()))
-        self.patchobject(stack_lock.StackLock, 'engine_alive',
+        self.patchobject(service_utils, 'engine_alive',
                          return_value=True)
         self.man.listener = mock.Mock()
         self.man.listener.SEND = 'send'
@@ -398,6 +520,23 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                           self.man.stack_cancel_update,
                           self.ctx, stk.identifier())
 
+    def test_stack_cancel_update_no_lock(self):
+        stack_name = 'service_update_stack_test_cancel_same_engine'
+        stk = tools.get_stack(stack_name, self.ctx)
+        stk.state_set(stk.UPDATE, stk.IN_PROGRESS, 'test_override')
+        stk.disable_rollback = False
+        stk.store()
+
+        self.patchobject(stack.Stack, 'load', return_value=stk)
+        self.patchobject(stack_lock.StackLock, 'get_engine_id',
+                         return_value=None)
+        self.patchobject(self.man.thread_group_mgr, 'send')
+
+        self.man.stack_cancel_update(self.ctx, stk.identifier(),
+                                     cancel_with_rollback=False)
+
+        self.assertFalse(self.man.thread_group_mgr.send.called)
+
     def test_stack_cancel_update_wrong_state_fails(self):
         stack_name = 'service_update_cancel_test_stack'
         stk = tools.get_stack(stack_name, self.ctx)
@@ -444,7 +583,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_validate = self.patchobject(stk, 'validate', return_value=None)
 
         # do update
-        cfg.CONF.set_override('max_resources_per_stack', 3)
+        cfg.CONF.set_override('max_resources_per_stack', 3, enforce_type=True)
 
         api_args = {'timeout_mins': 60}
         result = self.man.update_stack(self.ctx, old_stack.identifier(),
@@ -457,7 +596,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         root_stack_id = old_stack.root_stack_id()
         self.assertEqual(3, old_stack.total_resources(root_stack_id))
 
-        mock_tmpl.assert_called_once_with(template, files=None, env=stk.env)
+        mock_tmpl.assert_called_once_with(template, files=None)
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
@@ -532,7 +671,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         sid = old_stack.store()
         self.assertIsNotNone(sid)
 
-        cfg.CONF.set_override('max_resources_per_stack', 2)
+        cfg.CONF.set_override('max_resources_per_stack', 2, enforce_type=True)
 
         ex = self.assertRaises(dispatcher.ExpectedException,
                                self.man.update_stack, self.ctx,
@@ -573,7 +712,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
 
         # assertions
         self.assertEqual(exception.StackValidationFailed, ex.exc_info[0])
-        mock_tmpl.assert_called_once_with(template, files=None, env=stk.env)
+        mock_tmpl.assert_called_once_with(template, files=None)
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
@@ -636,7 +775,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
 
         mock_get.assert_called_once_with(self.ctx, stk.identifier())
 
-        mock_tmpl.assert_called_once_with(template, files=None, env=stk.env)
+        mock_tmpl.assert_called_once_with(template, files=None)
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
@@ -723,7 +862,6 @@ parameters:
 
         self.ctx = utils.dummy_context(password=None)
         stack_name = 'test_update_immutable_parameters'
-        params = {}
         old_stack = tools.get_stack(stack_name, self.ctx,
                                     template=template)
         sid = old_stack.store()
@@ -734,15 +872,12 @@ parameters:
         self.patchobject(self.man, '_get_stack', return_value=s)
         self.patchobject(stack, 'Stack', return_value=old_stack)
         self.patchobject(stack.Stack, 'load', return_value=old_stack)
-        self.patchobject(templatem, 'Template', return_value=old_stack.t)
-        self.patchobject(environment, 'Environment',
-                         return_value=old_stack.env)
 
         params = {'param1': 'bar'}
         exc = self.assertRaises(dispatcher.ExpectedException,
                                 self.man.update_stack,
                                 self.ctx, old_stack.identifier(),
-                                templatem.Template(template), params,
+                                old_stack.t.t, params,
                                 None, {})
         self.assertEqual(exception.ImmutableParameterModified, exc.exc_info[0])
         self.assertEqual('The following parameters are immutable and may not '
@@ -881,7 +1016,7 @@ resources:
         mock_env = self.patchobject(environment, 'Environment',
                                     return_value=stk.env)
         mock_validate = self.patchobject(stk, 'validate', return_value=None)
-        mock_merge = self.patchobject(self.man, '_merge_environments')
+        mock_merge = self.patchobject(env_util, 'merge_environments')
 
         # Patch _resolve_all_attributes or it tries to call novaclient
         self.patchobject(resource.Resource, '_resolve_all_attributes',
@@ -906,13 +1041,13 @@ resources:
             strict_validate=True, tenant_id='test_tenant_id', timeout_mins=60,
             user_creds_id=u'1', username='test_username')
         mock_load.assert_called_once_with(self.ctx, stack=s)
-        mock_tmpl.assert_called_once_with(new_template, files=None,
-                                          env=stk.env)
+        mock_tmpl.assert_called_once_with(new_template, files=None)
         mock_env.assert_called_once_with(params)
         mock_validate.assert_called_once_with()
 
         if environment_files:
-            mock_merge.assert_called_once_with(environment_files, None, params)
+            mock_merge.assert_called_once_with(environment_files, None,
+                                               params, mock.ANY)
 
         return result
 
@@ -990,3 +1125,41 @@ resources:
                                         environment_files=environment_files)
 
         # Assertions done in _test_stack_update_preview
+
+    def test_reset_stack_and_resources_in_progress(self):
+
+        def mock_stack_resource(name, action, status):
+            rs = mock.MagicMock()
+            rs.name = name
+            rs.action = action
+            rs.status = status
+            rs.IN_PROGRESS = 'IN_PROGRESS'
+            rs.FAILED = 'FAILED'
+
+            def mock_resource_state_set(a, s, reason='engine_down'):
+                rs.status = s
+                rs.action = a
+                rs.status_reason = reason
+
+            rs.state_set = mock_resource_state_set
+
+            return rs
+
+        stk_name = 'test_stack'
+        stk = tools.get_stack(stk_name, self.ctx)
+        stk.action = 'CREATE'
+        stk.status = 'IN_PROGRESS'
+
+        resources = {'r1': mock_stack_resource('r1', 'UPDATE', 'COMPLETE'),
+                     'r2': mock_stack_resource('r2', 'UPDATE', 'IN_PROGRESS'),
+                     'r3': mock_stack_resource('r3', 'UPDATE', 'FAILED')}
+
+        stk._resources = resources
+
+        reason = 'Test resetting stack and resources in progress'
+
+        stk.reset_stack_and_resources_in_progress(reason)
+        self.assertEqual('FAILED', stk.status)
+        self.assertEqual('COMPLETE', stk.resources.get('r1').status)
+        self.assertEqual('FAILED', stk.resources.get('r2').status)
+        self.assertEqual('FAILED', stk.resources.get('r3').status)
