@@ -33,6 +33,8 @@ iniset $conf_file heat_plugin username $OS_USERNAME
 iniset $conf_file heat_plugin password $OS_PASSWORD
 iniset $conf_file heat_plugin tenant_name $OS_PROJECT_NAME
 iniset $conf_file heat_plugin auth_url $OS_AUTH_URL
+iniset $conf_file heat_plugin user_domain_id $OS_USER_DOMAIN_ID
+iniset $conf_file heat_plugin project_domain_id $OS_PROJECT_DOMAIN_ID
 iniset $conf_file heat_plugin user_domain_name $OS_USER_DOMAIN_NAME
 iniset $conf_file heat_plugin project_domain_name $OS_PROJECT_DOMAIN_NAME
 iniset $conf_file heat_plugin region $OS_REGION_NAME
@@ -61,8 +63,12 @@ iniset $conf_file heat_plugin boot_config_env $DEST/heat-templates/hot/software-
 iniset $conf_file heat_plugin heat_config_notify_script $DEST/heat-templates/hot/software-config/elements/heat-config/bin/heat-config-notify
 iniset $conf_file heat_plugin minimal_image_ref cirros-0.3.4-x86_64-uec
 
+# Skip test_cancel_update_server_with_port till bug #1607714 is fixed in nova
+iniset $conf_file heat_plugin skip_functional_test_list 'CancelUpdateTest.test_cancel_update_server_with_port'
+
 # Add scenario tests to skip
 # VolumeBackupRestoreIntegrationTest skipped until failure rate can be reduced ref bug #1382300
-iniset $conf_file heat_plugin skip_scenario_test_list 'SoftwareConfigIntegrationTest, VolumeBackupRestoreIntegrationTest'
+# test_server_signal_userdata_format_software_config is skipped untill bug #1651768 is resolved
+iniset $conf_file heat_plugin skip_scenario_test_list 'SoftwareConfigIntegrationTest, VolumeBackupRestoreIntegrationTest, ServerSignalIntegrationTest.test_server_signal_userdata_format_software_config'
 
 cat $conf_file
